package com.reyax.iotdemoplus2;

import android.content.Context;
import android.util.Log;

import org.eclipse.paho.android.service.MqttAndroidClient;
import org.eclipse.paho.client.mqttv3.DisconnectedBufferOptions;
import org.eclipse.paho.client.mqttv3.IMqttActionListener;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.IMqttToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

import java.util.Random;


public class MQTTHelper {
    public static MqttAndroidClient mqttAndroidClient;

    static String serverUri;
    static String username;
    static String password;;
    static String networkId;

    static String cmdSubTopic;
    static String cmdUpdSubTopic;
    static String notiSubTopic;

    static String PubTopic = "api/request";


    public MQTTHelper(Context context){

        Random generator = new Random();
        String id = String.valueOf(generator.nextInt(9999-2));

        String clientId = username +"-"+id;

        cmdSubTopic = "api/command/"+networkId+"/#";
        cmdUpdSubTopic = "api/command_update/"+networkId+"/#";
        notiSubTopic = "api/notification/"+networkId+"/#";


        mqttAndroidClient = new MqttAndroidClient(context, serverUri, clientId);
        mqttAndroidClient.setCallback(new MqttCallbackExtended() {
            @Override
            public void connectComplete(boolean b, String s) {
                //subscribeToTopic();
                //Log.w("Mqtt", "connect to: " + serverUri + " OK");
                //Log.w("mqtt", s);
            }

            @Override
            public void connectionLost(Throwable throwable) {

            }

            @Override
            public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception {
                Log.w("Mqtt", mqttMessage.toString());
            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

            }
        });
        connect();
    }

    public void setCallback(MqttCallbackExtended callback) {
        mqttAndroidClient.setCallback(callback);
    }

    public void connect(){

        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
        mqttConnectOptions.setAutomaticReconnect(true);
        //Must set Clean Session true for mqtt broker
        mqttConnectOptions.setCleanSession(true);
        mqttConnectOptions.setUserName(username);
        mqttConnectOptions.setPassword(password.toCharArray());
        Log.w("Mqtt", "connecting to: " + serverUri );

        try {

            mqttAndroidClient.connect(mqttConnectOptions, null, new IMqttActionListener() {

                @Override
                public void onSuccess(IMqttToken asyncActionToken) {

                    DisconnectedBufferOptions disconnectedBufferOptions = new DisconnectedBufferOptions();
                    disconnectedBufferOptions.setBufferEnabled(true);
                    disconnectedBufferOptions.setBufferSize(100);
                    disconnectedBufferOptions.setPersistBuffer(false);
                    disconnectedBufferOptions.setDeleteOldestMessages(false);
                    mqttAndroidClient.setBufferOpts(disconnectedBufferOptions);
                    //subscribeToTopic();
                    Log.w("Mqtt", "set disBuf OK");

                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    Log.w("Mqtt", "Failed to connect to: " + serverUri + exception.toString());
                }
            });


        } catch (MqttException ex){
            ex.printStackTrace();
        }
    }


    //private void subscribeToTopic() {
    public static void subscribeToTopic() {
        try {

            mqttAndroidClient.subscribe(cmdUpdSubTopic, 0, null, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken asyncActionToken) {
                    Log.w("Mqtt","Command_update Subscribed!");
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    Log.w("Mqtt", "Command_update Subscribed fail!");
                }
            });

            mqttAndroidClient.subscribe(cmdSubTopic, 0, null, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken asyncActionToken) {
                    Log.w("Mqtt","Command Subscribed!");
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    Log.w("Mqtt", "Command Subscribed fail!");
                }
            });

            mqttAndroidClient.subscribe(notiSubTopic, 0, null, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken asyncActionToken) {
                    Log.w("Mqtt","Notification Subscribed!");
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    Log.w("Mqtt", "Notification Subscribed fail!");
                }
            });


        } catch (MqttException ex) {
            System.err.println("Exceptionst subscribing");
            ex.printStackTrace();
        }
    }

    public static void startPub(String m){

        try {

            Log.w("pubdeg",m);

            MqttMessage message = new MqttMessage(m.getBytes());message.setQos(0);

            mqttAndroidClient.publish(PubTopic, message);
        } catch (MqttException e) {
            e.printStackTrace();
        }
    }

    public static void disconnect(){

        try {
            mqttAndroidClient.disconnect();
            Log.w("Mqtt", "disconnect to: " + serverUri );
        } catch (MqttException e) {
            e.printStackTrace();
        }
    }

    public static boolean isConnect(){

            return mqttAndroidClient.isConnected();
    }

    public static String getServerUri() {
        return serverUri;
    }

    public static void setServerUri(String serverUri) {
        MQTTHelper.serverUri = serverUri;
    }

    public static String getUsername() {
        return username;
    }

    public static void setUsername(String username) {
        MQTTHelper.username = username;
    }

    public static String getPassword() {
        return password;
    }

    public static void setPassword(String password) {
        MQTTHelper.password = password;
    }

    public static String getNetworkId() {
        return networkId;
    }

    public static void setNetworkId(String networkId) {
        MQTTHelper.networkId = networkId;
    }
}
